@@ -219,8 +219,8 @@ sub CheckForLibGcc {
     # We try to fix this by always linking against libgcc.a. Unfortunately
     # it's somewhat hard to find out the path of this file ...
     #
-    if ($Config::Config{'osname'} eq 'netbsd') {
-	# Under NetBSD libc is nothing else than libgcc
+    if ($Config::Config{'osname'} eq '(free|net)bsd') {
+	# Under NetBSD and FreeBSD libc is nothing else than libgcc
 	return '';
     }
     if ($Config::Config{'gccversion'} eq '') {
@@ -344,19 +344,22 @@ sub Initialize ($) {
     }
 
     my $sysliblist;
+    $self->{'static_libs'} = '';
+    $self->{'final_libs'} = '';
     if ($::options->{'static'}) {
 	if ($libfile !~ /\.a/  &&  $libfile !~ /\.lib/) {
 	    die "Cannot use option -static with shared library file $libfile";
 	}
-	$self->{'static_libs'} = $libfile;
+	$self->{'static_libs'} .= " $libfile";
     } else {
 	$sysliblist = "-L$libdir -lmysqlclient";
     }
     $sysliblist .= " -lm";
     my $defines = "-DDBD_MYSQL";
     my $linkwith = "";
-    if ($Config::Config{'osname'} eq 'sco_sv') {
-	# Some extra libraries need added for SCO
+    if ($Config::Config{'osname'} eq 'sco_sv'  ||
+	$Config::Config{'osname'} eq 'svr4') {
+	# Some extra libraries need added for SCO and Unixware
 	$sysliblist .= " -lc";
     } elsif ($Config::Config{'osname'} eq 'solaris') {
 	# We need to link with -R if we're on Solaris.......Brain-damaged....
@@ -385,17 +388,21 @@ sub Initialize ($) {
 	if (! -f "$libdir/mysqlclient.lib"  &&  -f "$libdir/lib.lib") {
 	    # Looks like we're using MyODBC.
 	    $sysliblist =~ s/-lmysqlclient/-llib/;
-	    $sysliblist .= " -lmysys -lstrings -lzlib";
+	    $sysliblist .= " -lmysys -lstrings";
 	    $defines .= " -DWIN32";
 	}
+	if (-f "$libdir/zlib.lib") {
+	    $sysliblist .= " -lzlib";
+	}
 
 	# For some reasons libc.lib is not linked against msvcrt.lib
 	# with VC++ ...
-	if ($Config::Config{'libs'} =~ /\bmsvcrt\.lib\b/) {
-	    $sysliblist .= " -lmsvcrt";
-	}
-	if ($Config::Config{'libs'} =~ /\blibc\.lib\b/) {
-	    $sysliblist .= " -llibc";
+	if ($Config::Config{libs} =~ /\bmsvcrt\.lib\b/) {
+	    if ($Config::Config{libs} !~ /\blibc.lib\b/) {
+	        $sysliblist .= " -lmsvcrt -lc";
+	    }
+	} elsif ($Config::Config{libs} =~ /\bPerlCRT\.lib\b/) {
+	    $self->{'final_libs'} .= " libc.lib";
 	}
     }
 
