@@ -16,7 +16,7 @@ use DynaLoader();
 use Carp ();
 @ISA = qw(DynaLoader);
 
-$VERSION = '2.03_07';
+$VERSION = '2.03_08';
 
 bootstrap DBD::mSQL $VERSION;
 
@@ -113,7 +113,7 @@ sub connect {
     # create a 'blank' dbh
     my($this, $privateAttrHash);
     $privateAttrHash = {
-	'dsn' => $dsn,
+	'Name' => $dsn,
 	'user' => $username,
 	'password' => $password
     };
